<?php

namespace FiloBlu\Core\Helper;

use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Filesystem;
use Magento\PageBuilder\Controller\Adminhtml\ContentType\Image\Upload;
use Throwable;


class WysiwygOptimizer
{

    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var Filesystem
     */
    private $filesystem;
    /**
     * @var array
     */
    private $contentProviders;
    /**
     * @var array
     */
    private $contentModifiers;

    public function __construct(
        ResourceConnection $resourceConnection,
        Filesystem $filesystem,
        array  $contentProviders,
        array $contentModifiers
    ){
        $this->resourceConnection = $resourceConnection;
        $this->filesystem = $filesystem;
        $this->contentProviders = $contentProviders;
        $this->contentModifiers = $contentModifiers;
    }

    public function getUsedImages(){
        $imagesArray = [];

        foreach ($this->contentProviders as $provider){
            foreach ($provider->getContents() as $content){
                preg_match_all('/\"(wysiwyg\/.*)\"/U', $content, $output);
                $imagesArray = array_merge($imagesArray, $output[1]);
            }
        }
        return array_unique($imagesArray);

    }

    public function checkFiles($fileList){
        $mediapath = $this->filesystem->getDirectoryRead(DirectoryList::MEDIA);
        $map = [];
        $preservedImage = [];
        $preservedAbsoluteImages = [];
        foreach ($fileList as $file){
            $absoluteFilePath = $mediapath->getAbsolutePath($file);
            try{
                $md5 = md5_file($absoluteFilePath);
            } catch (Throwable $exception){
                continue;
            }

            if(!array_key_exists($md5, $map)){
                $map[$md5] = [];
                $preservedImage[$md5] = $file;
                $preservedAbsoluteImages[$md5] = $absoluteFilePath;
                continue;
            }
            $map[$md5][] = $file;
        }

        return ['duplicates' => array_filter($map), 'preserved' => $preservedImage, 'preserved_absolute' => $preservedAbsoluteImages];
    }

    public function moveFiles($dataList){
        $wysiwygDir = $this->filesystem->getDirectoryRead(DirectoryList::MEDIA)->getAbsolutePath(Upload::UPLOAD_DIR);
        foreach (glob("$wysiwygDir/*")as $file){
            if(in_array($file, $dataList['preserved_absolute'])){
                continue;
            }
            $this->filesystem->getDirectoryWrite(DirectoryList::MEDIA)->renameFile($file, str_replace(Upload::UPLOAD_DIR, Upload::UPLOAD_DIR . '/backup', $file));
        }

        return true;
    }

    public function fixDuplicatedContentImage($dataList){
        $arrayToReplace = [];

        foreach($dataList['duplicates'] as $hash => $duplicates){

            $filename = $dataList['preserved'][$hash];
            foreach($duplicates as $duplicate){
                echo "{$duplicate} => {$filename}\n";
                $arrayToReplace[] = ['from' => $duplicate, 'to' => $dataList['preserved'][$hash]];
            }
        }
        foreach ($this->contentModifiers as $modifier){
            $modifier->updateContent($arrayToReplace);
        }
    }
}
